package de.fraunhofer.iem.dataprovider.toolRun.entity

import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.tool.dto.ToolDto
import de.fraunhofer.iem.dataprovider.tool.entity.ToolEntity
import de.fraunhofer.iem.dataprovider.toolRun.dto.ToolRunDto
import jakarta.persistence.*
import org.hibernate.annotations.CurrentTimestamp
import org.hibernate.annotations.UpdateTimestamp
import org.hibernate.generator.EventType
import java.time.Instant
import java.util.*

@Entity
@Table(name = "tool_run")
class ToolRunEntity(
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null,

    @ManyToOne(fetch = FetchType.EAGER)
    val repository: RepositoryEntity,

    @ManyToMany
    @JoinTable(
        name = "tool_run_tool_entities",
        joinColumns = [JoinColumn(name = "tool_run_entity_id")],
        inverseJoinColumns = [JoinColumn(name = "tool_entities_id")]
    )
    val toolEntities: MutableSet<ToolEntity> = mutableSetOf(),

    @CurrentTimestamp(event = [EventType.INSERT])
    var createdAt: Instant? = null,

    @UpdateTimestamp
    var lastUpdatedAt: Instant? = null
) {
    fun toDto(): ToolRunDto {
        if (this.id != null) {
            // TODO: once we store findings this should be added here
            return ToolRunDto(
                id = this.id!!,
                repoId = this.repository.repoId,
                createdAt = this.createdAt?.toString() ?: Instant.now().toString(),
                tools = toolEntities.map { ToolDto(name = it.name) }
            )
        } else {
            throw Exception("Tool run DTO can't be created for incomplete entity. ID is null.")
        }
    }
}
